/**
 * BLUECARAT AG
 *
 * values at work.
 *
 * Albin-Köbis-Straße 4
 * 51147 Köln
 *
 * http://www.bluecarat.de
 *
 * This module is free software, and you may redistribute it and/or modify it under the same terms as Jenkins itself.
 */
package de.bluecarat.trafficlight.connectionhandler;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;

import de.bluecarat.trafficlight.connectionhandler.exception.HandlerException;

/**
 * This class is responsible for HTTP Post handling and timeout issues. It submits the content via HTTP Post.
 *
 * @author STH
 * @author DKE
 *
 */
public final class HttpPostHandler extends BaseHttpHandler {

    /**
     * Logger for this class.
     */
    private static final Logger LOGGER = Logger.getLogger(HttpPostHandler.class.getName());

    /**
     * Create a new get handler instance.
     *
     * @param timeoutForConnections
     *            for Get and Post requests
     */
    public HttpPostHandler(final int timeoutForConnections) {
        super(timeoutForConnections);
    }

    /**
     * Create a new get handler instance with authentication.
     *
     * @param username
     *            to use for authentication
     * @param password
     *            to use for authentication
     * @param timeoutForConnections
     *            for Get and Post requests
     */
    public HttpPostHandler(final String username, final String password, final int timeoutForConnections) {
        super(username, password, timeoutForConnections);
    }

    /**
     * Execute HTTP-GET request for the given URI.
     *
     * @param uri
     *            to execute a post request to
     * @param postBody
     *            data to submit
     * @throws HandlerException
     *             In case an error occurs
     */
    public void handle(final URI uri, final String postBody) throws HandlerException {
        submit(uri, postBody);
        LOGGER.log(Level.FINEST, "Request sucessfully.");
    }

    private void submit(final URI uri, final String postBody) throws HandlerException {
        LOGGER.log(Level.FINE, "Executing POST request for " + uri.toString() + ".");

        final PostMethod postMethod = new PostMethod(uri.toString());
        setPostBody(uri, postBody, postMethod);

        try {
            final int postResult = getHttpclient().executeMethod(postMethod);

            if (postResult != HttpStatus.SC_OK) {
                throw new HandlerException("Unexpected response, aborting (" + uri.getHost() + "). Status code: "
                        + postResult);
            }
        } catch (HttpException e) {
            throwGenericHandlerException(uri, e);
        } catch (IOException e) {
            throwGenericHandlerException(uri, e);
        }
    }

    private void throwGenericHandlerException(final URI uri, final Exception e) throws HandlerException {
        throw new HandlerException("Unexpected Error, aborting (" + uri.getHost() + ").", e);
    }

    private void setPostBody(final URI uri, final String body, final PostMethod method) throws HandlerException {
        try {
            method.setRequestEntity(new StringRequestEntity(body, "text/html", "utf-8"));
        } catch (final UnsupportedEncodingException e) {
            throwGenericHandlerException(uri, e);
        }
    }
}
